/* Copyright 2010 Ralph Gauges
 *
 * This library is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation.  A copy of the license agreement is
 * provided in the file named "LICENSE.txt" included with this software
 * distribution.  It is also available online at
 * http://www.gnu.org/licenses/lgpl.html
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with this library; if not, write to the Free Software Foundation,
 * Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA.
 *
 * The original code contained here was initially developed by:
 *
 *     Ralph Gauges
 *     Group for the modeling of biological processes 
 *     University of Heidelberg
 *     Im Neuenheimer Feld 267
 *     69120 Heidelberg
 *     Germany
 *
 *     mailto:ralph.gauges@bioquant.uni-heidelberg.de
 *
 * Contributor(s):
 *
 */

#include <sbml/SBMLDocument.h>
#include <sbml/SBMLWriter.h>
#include <sbml/Model.h>
#include <sbml/layout/Layout.h>
#include <sbml/layout/BoundingBox.h>
#include <sbml/layout/Point.h>
#include <sbml/layout/Dimensions.h>
#include <sbml/layout/GraphicalObject.h>
#include <sbml/layout/render/LocalRenderInformation.h>
#include <sbml/layout/render/GlobalRenderInformation.h>
#include <sbml/layout/render/LocalStyle.h>
#include <sbml/layout/render/GlobalStyle.h>
#include <sbml/layout/render/Rectangle.h>
#include <sbml/layout/render/LinearGradient.h>
#include <sbml/layout/render/GradientStop.h>

#include <cmath>
#include <sstream>
#include <string>
#include <stdlib.h>
#include <iostream>

using namespace std;

/**
 * This example draws three rectangles.
 * One with both sides having the same length, one
 * where the width is larger and one where the height is larger.
 * All three rectangles are filled with a single color and have no outline.
 */

int main(int argc,char** argv)
{
    // first we create a new SBML Document with a model
    unsigned int level=SBML_DEFAULT_LEVEL;
    unsigned int version=SBML_DEFAULT_VERSION;
    SBMLDocument* pDocument=NULL;
    if(argc != 1 && argc != 3)
    {
        std::cout << "usage: " << argv[0] << " [LEVEL VERSION]" << std::endl;
        std::cout << "e.g. " << argv[0] << " 2 3" << std::endl;
        std::cout << "or " << argv[0] << std::endl;
        return 0;
    }
    else
    {
       if(argc == 1)
       {
           pDocument=new SBMLDocument(level,version);
       }
       else
       {
         level=strtol(argv[1],NULL,10); 
         version=strtol(argv[2],NULL,10); 
         try{
           pDocument=new SBMLDocument(level,version);
           if(!pDocument->hasValidLevelVersionNamespaceCombination())
           {
             std::cerr << "Invalid Level and/or version. Aborting." << std::endl;
             return 1;
           }
         }
         catch(...)
         {
            std::cerr << "Invalid Level and/or version. Aborting." << std::endl;
            return 1;
         }
       }

    }
    assert(pDocument != NULL);
    Model* pModel=pDocument->createModel("Model_1");
    assert(pModel != NULL);
    if(level > 2)
    {
       UnitDefinition* pUDef=pModel->createUnitDefinition();
       pUDef->setId("squaremetre");
       pUDef->setName("squaremetre");
       Unit* pUnit=pUDef->createUnit();
       pUnit->setKind(UNIT_KIND_METRE);
       pUnit->setMultiplier(1.0);
       pUnit->setScale(0);
       pUnit->setExponent(2.0);
       pModel->setTimeUnits("second"); 
       pModel->setSubstanceUnits("mole"); 
       pModel->setVolumeUnits("litre"); 
       pModel->setAreaUnits("squaremetre"); 
       pModel->setLengthUnits("metre"); 
       pModel->setExtentUnits("mole"); 
    }

    // now we create a new layout
    Layout* pLayout=pModel->createLayout();
    pLayout->setId("GradientTest");
    const double MARGIN_X=5.0;
    const double MARGIN_Y=5.0;
    // draw each line in a 400x100 box
    const double WIDTH=400;
    const double HEIGHT=100;
    double x=MARGIN_X;
    double y=MARGIN_Y;
    ostringstream os;
    unsigned int object_index=1;
    // the dimensions of the whole layout will be
    BoundingBox bb;
    Point p;
    Dimensions dim(2*WIDTH*1.2+2*MARGIN_X,6*HEIGHT*1.2+2*MARGIN_Y);
    pLayout->setDimensions(&dim);
    // create the render information
    LocalRenderInformation* pRenderInfo=pLayout->createLocalRenderInformation();
    LinearGradient* pLG=NULL;
    GradientStop* pGS=NULL;
    pRenderInfo->setId("GradientRenderInfo");
    Dimensions d(WIDTH,HEIGHT);
   
    // create the gradients
    // padded gradient from red to blue
    pLG=pRenderInfo->createLinearGradientDefinition();
    pLG->setId("linear_gradient_pad");
    pLG->setSpreadMethod(GradientBase::PAD);
    pLG->setPoint1(RelAbsVector(0.0,30.0),RelAbsVector(0.0,0.0),RelAbsVector(0.0,0.0));
    pLG->setPoint2(RelAbsVector(0.0,70.0),RelAbsVector(0.0,0.0),RelAbsVector(0.0,0.0));
    pGS=pLG->createGradientStop();
    pGS->setOffset(RelAbsVector(0.0,20.0));
    pGS->setStopColor("#FF0000");
    pGS=pLG->createGradientStop();
    pGS->setOffset(RelAbsVector(0.0,80.0));
    pGS->setStopColor("#0000FF");

    // reflected gradient from red to blue
    pLG=pRenderInfo->createLinearGradientDefinition();
    pLG->setId("linear_gradient_reflect");
    pLG->setSpreadMethod(GradientBase::REFLECT);
    pLG->setPoint1(RelAbsVector(0.0,30.0),RelAbsVector(0.0,0.0),RelAbsVector(0.0,0.0));
    pLG->setPoint2(RelAbsVector(0.0,70.0),RelAbsVector(0.0,0.0),RelAbsVector(0.0,0.0));
    pGS=pLG->createGradientStop();
    pGS->setOffset(RelAbsVector(0.0,20.0));
    pGS->setStopColor("#FF0000");
    pGS=pLG->createGradientStop();
    pGS->setOffset(RelAbsVector(0.0,80.0));
    pGS->setStopColor("#0000FF");

    // repeated gradient from red to blue
    pLG=pRenderInfo->createLinearGradientDefinition();
    pLG->setId("linear_gradient_repeat");
    pLG->setSpreadMethod(GradientBase::REPEAT);
    pLG->setPoint1(RelAbsVector(0.0,30.0),RelAbsVector(0.0,0.0),RelAbsVector(0.0,0.0));
    pLG->setPoint2(RelAbsVector(0.0,70.0),RelAbsVector(0.0,0.0),RelAbsVector(0.0,0.0));
    pGS=pLG->createGradientStop();
    pGS->setOffset(RelAbsVector(0.0,20.0));
    pGS->setStopColor("#FF0000");
    pGS=pLG->createGradientStop();
    pGS->setOffset(RelAbsVector(0.0,80.0));
    pGS->setStopColor("#0000FF");

    // diagonal padded gradient from red to blue
    pLG=pRenderInfo->createLinearGradientDefinition();
    pLG->setId("linear_gradient_diag_pad");
    pLG->setSpreadMethod(GradientBase::PAD);
    pLG->setPoint1(RelAbsVector(0.0,30.0),RelAbsVector(0.0,10.0),RelAbsVector(0.0,0.0));
    pLG->setPoint2(RelAbsVector(0.0,70.0),RelAbsVector(0.0,90.0),RelAbsVector(0.0,0.0));
    pGS=pLG->createGradientStop();
    pGS->setOffset(RelAbsVector(0.0,20.0));
    pGS->setStopColor("#FF0000");
    pGS=pLG->createGradientStop();
    pGS->setOffset(RelAbsVector(0.0,80.0));
    pGS->setStopColor("#0000FF");

    // diagonal reflected gradient from red to blue
    pLG=pRenderInfo->createLinearGradientDefinition();
    pLG->setId("linear_gradient_diag_reflect");
    pLG->setSpreadMethod(GradientBase::REFLECT);
    pLG->setPoint1(RelAbsVector(0.0,30.0),RelAbsVector(0.0,10.0),RelAbsVector(0.0,0.0));
    pLG->setPoint2(RelAbsVector(0.0,70.0),RelAbsVector(0.0,90.0),RelAbsVector(0.0,0.0));
    pGS=pLG->createGradientStop();
    pGS->setOffset(RelAbsVector(0.0,20.0));
    pGS->setStopColor("#FF0000");
    pGS=pLG->createGradientStop();
    pGS->setOffset(RelAbsVector(0.0,80.0));
    pGS->setStopColor("#0000FF");

    // diagonal repeated gradient from red to blue
    pLG=pRenderInfo->createLinearGradientDefinition();
    pLG->setId("linear_gradient_diag_repeat");
    pLG->setPoint1(RelAbsVector(0.0,30.0),RelAbsVector(0.0,10.0),RelAbsVector(0.0,0.0));
    pLG->setPoint2(RelAbsVector(0.0,70.0),RelAbsVector(0.0,90.0),RelAbsVector(0.0,0.0));
    pLG->setSpreadMethod(GradientBase::REPEAT);
    pGS=pLG->createGradientStop();
    pGS->setOffset(RelAbsVector(0.0,20.0));
    pGS->setStopColor("#FF0000");
    pGS=pLG->createGradientStop();
    pGS->setOffset(RelAbsVector(0.0,80.0));
    pGS->setStopColor("#0000FF");

    // padded gradient from red to blue
    pLG=pRenderInfo->createLinearGradientDefinition();
    pLG->setId("linear_gradient_pad_narrow");
    pLG->setSpreadMethod(GradientBase::PAD);
    pLG->setPoint1(RelAbsVector(0.0,45.0),RelAbsVector(0.0,0.0),RelAbsVector(0.0,0.0));
    pLG->setPoint2(RelAbsVector(0.0,55.0),RelAbsVector(0.0,0.0),RelAbsVector(0.0,0.0));
    pGS=pLG->createGradientStop();
    pGS->setOffset(RelAbsVector(0.0,20.0));
    pGS->setStopColor("#FF0000");
    pGS=pLG->createGradientStop();
    pGS->setOffset(RelAbsVector(0.0,80.0));
    pGS->setStopColor("#0000FF");

    // reflected gradient from red to blue
    pLG=pRenderInfo->createLinearGradientDefinition();
    pLG->setId("linear_gradient_reflect_narrow");
    pLG->setSpreadMethod(GradientBase::REFLECT);
    pLG->setPoint1(RelAbsVector(0.0,45.0),RelAbsVector(0.0,0.0),RelAbsVector(0.0,0.0));
    pLG->setPoint2(RelAbsVector(0.0,55.0),RelAbsVector(0.0,0.0),RelAbsVector(0.0,0.0));
    pGS=pLG->createGradientStop();
    pGS->setOffset(RelAbsVector(0.0,20.0));
    pGS->setStopColor("#FF0000");
    pGS=pLG->createGradientStop();
    pGS->setOffset(RelAbsVector(0.0,80.0));
    pGS->setStopColor("#0000FF");

    // repeated gradient from red to blue
    pLG=pRenderInfo->createLinearGradientDefinition();
    pLG->setId("linear_gradient_repeat_narrow");
    pLG->setSpreadMethod(GradientBase::REPEAT);
    pLG->setPoint1(RelAbsVector(0.0,45.0),RelAbsVector(0.0,0.0),RelAbsVector(0.0,0.0));
    pLG->setPoint2(RelAbsVector(0.0,55.0),RelAbsVector(0.0,0.0),RelAbsVector(0.0,0.0));
    pGS=pLG->createGradientStop();
    pGS->setOffset(RelAbsVector(0.0,20.0));
    pGS->setStopColor("#FF0000");
    pGS=pLG->createGradientStop();
    pGS->setOffset(RelAbsVector(0.0,80.0));
    pGS->setStopColor("#0000FF");

    // diagonal padded gradient from red to blue
    pLG=pRenderInfo->createLinearGradientDefinition();
    pLG->setId("linear_gradient_diag_pad_narrow");
    pLG->setSpreadMethod(GradientBase::PAD);
    pLG->setPoint1(RelAbsVector(0.0,45.0),RelAbsVector(0.0,45.0),RelAbsVector(0.0,0.0));
    pLG->setPoint2(RelAbsVector(0.0,55.0),RelAbsVector(0.0,55.0),RelAbsVector(0.0,0.0));
    pGS=pLG->createGradientStop();
    pGS->setOffset(RelAbsVector(0.0,20.0));
    pGS->setStopColor("#FF0000");
    pGS=pLG->createGradientStop();
    pGS->setOffset(RelAbsVector(0.0,80.0));
    pGS->setStopColor("#0000FF");

    // diagonal reflected gradient from red to blue
    pLG=pRenderInfo->createLinearGradientDefinition();
    pLG->setId("linear_gradient_diag_reflect_narrow");
    pLG->setSpreadMethod(GradientBase::REFLECT);
    pLG->setPoint1(RelAbsVector(0.0,45.0),RelAbsVector(0.0,45.0),RelAbsVector(0.0,0.0));
    pLG->setPoint2(RelAbsVector(0.0,55.0),RelAbsVector(0.0,55.0),RelAbsVector(0.0,0.0));
    pGS=pLG->createGradientStop();
    pGS->setOffset(RelAbsVector(0.0,20.0));
    pGS->setStopColor("#FF0000");
    pGS=pLG->createGradientStop();
    pGS->setOffset(RelAbsVector(0.0,80.0));
    pGS->setStopColor("#0000FF");

    // diagonal repeated gradient from red to blue
    pLG=pRenderInfo->createLinearGradientDefinition();
    pLG->setId("linear_gradient_diag_repeat_narrow");
    pLG->setPoint1(RelAbsVector(0.0,45.0),RelAbsVector(0.0,45.0),RelAbsVector(0.0,0.0));
    pLG->setPoint2(RelAbsVector(0.0,55.0),RelAbsVector(0.0,55.0),RelAbsVector(0.0,0.0));
    pLG->setSpreadMethod(GradientBase::REPEAT);
    pGS=pLG->createGradientStop();
    pGS->setOffset(RelAbsVector(0.0,20.0));
    pGS->setStopColor("#FF0000");
    pGS=pLG->createGradientStop();
    pGS->setOffset(RelAbsVector(0.0,80.0));
    pGS->setStopColor("#0000FF");

    GraphicalObject* pObject=NULL;
    LocalStyle* pStyle=NULL;
    Group* pGroup=NULL;
    Rectangle* pRectangle=NULL;

    // first rectangle
    // create a graphical object with the correct bounding box
    os.str("");
    os << "graphical_object_" << object_index++;
    pObject=pLayout->createAdditionalGraphicalObject();
    pObject->setId(os.str());
    // we set the bounding box
    p=Point(x,y);
    bb.setPosition(&p);
    bb.setDimensions(&d);
    pObject->setBoundingBox(&bb);
    // we create a style for the object
    pStyle=pRenderInfo->createStyle("style_for_"+os.str());
    pStyle->addId(pObject->getId());
    pGroup=pStyle->getGroup();
    pGroup->setFillColor("linear_gradient_pad");
    pGroup->setStroke("#000000");
    pGroup->setStrokeWidth(2.0);
    // add an rectangle to the group
    pRectangle=pGroup->createRectangle();
    pRectangle->setCoordinates(RelAbsVector(0.0,0.0),RelAbsVector(0.0,0.0),RelAbsVector(0.0,0.0));
    pRectangle->setSize(RelAbsVector(0.0,100),RelAbsVector(0.0,100));
    y+=1.2*HEIGHT;

    // second rectangle
    // create a graphical object with the correct bounding box
    os.str("");
    os << "graphical_object_" << object_index++;
    pObject=pLayout->createAdditionalGraphicalObject();
    pObject->setId(os.str());
    // we set the bounding box
    p=Point(x,y);
    bb.setPosition(&p);
    bb.setDimensions(&d);
    pObject->setBoundingBox(&bb);
    // we create a style for the object
    pStyle=pRenderInfo->createStyle("style_for_"+os.str());
    pStyle->addId(pObject->getId());
    pGroup=pStyle->getGroup();
    pGroup->setFillColor("linear_gradient_reflect");
    pGroup->setStroke("#000000");
    pGroup->setStrokeWidth(2.0);
    // add an rectangle to the group
    pRectangle=pGroup->createRectangle();
    pRectangle->setCoordinates(RelAbsVector(0.0,0.0),RelAbsVector(0.0,0.0),RelAbsVector(0.0,0.0));
    pRectangle->setSize(RelAbsVector(0.0,100),RelAbsVector(0.0,100));
    y+=1.2*HEIGHT;
    
    // third rectangle
    // create a graphical object with the correct bounding box
    os.str("");
    os << "graphical_object_" << object_index++;
    pObject=pLayout->createAdditionalGraphicalObject();
    pObject->setId(os.str());
    // we set the bounding box
    p=Point(x,y);
    bb.setPosition(&p);
    bb.setDimensions(&d);
    pObject->setBoundingBox(&bb);
    // we create a style for the object
    pStyle=pRenderInfo->createStyle("style_for_"+os.str());
    pStyle->addId(pObject->getId());
    pGroup=pStyle->getGroup();
    pGroup->setFillColor("linear_gradient_repeat");
    pGroup->setStroke("#000000");
    pGroup->setStrokeWidth(2.0);
    // add an rectangle to the group
    pRectangle=pGroup->createRectangle();
    pRectangle->setCoordinates(RelAbsVector(0.0,0.0),RelAbsVector(0.0,0.0),RelAbsVector(0.0,0.0));
    pRectangle->setSize(RelAbsVector(0.0,100),RelAbsVector(0.0,100));
    y+=1.2*HEIGHT;
    
    // fourth rectangle
    // create a graphical object with the correct bounding box
    os.str("");
    os << "graphical_object_" << object_index++;
    pObject=pLayout->createAdditionalGraphicalObject();
    pObject->setId(os.str());
    // we set the bounding box
    p=Point(x,y);
    bb.setPosition(&p);
    bb.setDimensions(&d);
    pObject->setBoundingBox(&bb);
    // we create a style for the object
    pStyle=pRenderInfo->createStyle("style_for_"+os.str());
    pStyle->addId(pObject->getId());
    pGroup=pStyle->getGroup();
    pGroup->setFillColor("linear_gradient_diag_pad");
    pGroup->setStroke("#000000");
    pGroup->setStrokeWidth(2.0);
    // add an rectangle to the group
    pRectangle=pGroup->createRectangle();
    pRectangle->setCoordinates(RelAbsVector(0.0,0.0),RelAbsVector(0.0,0.0),RelAbsVector(0.0,0.0));
    pRectangle->setSize(RelAbsVector(0.0,100),RelAbsVector(0.0,100));
    y+=1.2*HEIGHT;
    
    // fifth rectangle
    // create a graphical object with the correct bounding box
    os.str("");
    os << "graphical_object_" << object_index++;
    pObject=pLayout->createAdditionalGraphicalObject();
    pObject->setId(os.str());
    // we set the bounding box
    p=Point(x,y);
    bb.setPosition(&p);
    bb.setDimensions(&d);
    pObject->setBoundingBox(&bb);
    // we create a style for the object
    pStyle=pRenderInfo->createStyle("style_for_"+os.str());
    pStyle->addId(pObject->getId());
    pGroup=pStyle->getGroup();
    pGroup->setFillColor("linear_gradient_diag_reflect");
    pGroup->setStroke("#000000");
    pGroup->setStrokeWidth(2.0);
    // add an rectangle to the group
    pRectangle=pGroup->createRectangle();
    pRectangle->setCoordinates(RelAbsVector(0.0,0.0),RelAbsVector(0.0,0.0),RelAbsVector(0.0,0.0));
    pRectangle->setSize(RelAbsVector(0.0,100),RelAbsVector(0.0,100));
    y+=1.2*HEIGHT;
    
    // sixth rectangle
    // create a graphical object with the correct bounding box
    os.str("");
    os << "graphical_object_" << object_index++;
    pObject=pLayout->createAdditionalGraphicalObject();
    pObject->setId(os.str());
    // we set the bounding box
    p=Point(x,y);
    bb.setPosition(&p);
    bb.setDimensions(&d);
    pObject->setBoundingBox(&bb);
    // we create a style for the object
    pStyle=pRenderInfo->createStyle("style_for_"+os.str());
    pStyle->addId(pObject->getId());
    pGroup=pStyle->getGroup();
    pGroup->setFillColor("linear_gradient_diag_repeat");
    pGroup->setStroke("#000000");
    pGroup->setStrokeWidth(2.0);
    // add an rectangle to the group
    pRectangle=pGroup->createRectangle();
    pRectangle->setCoordinates(RelAbsVector(0.0,0.0),RelAbsVector(0.0,0.0),RelAbsVector(0.0,0.0));
    pRectangle->setSize(RelAbsVector(0.0,100),RelAbsVector(0.0,100));


    y=MARGIN_Y;
    x+=1.2*WIDTH;
    
    // seventh rectangle
    // create a graphical object with the correct bounding box
    os.str("");
    os << "graphical_object_" << object_index++;
    pObject=pLayout->createAdditionalGraphicalObject();
    pObject->setId(os.str());
    // we set the bounding box
    p=Point(x,y);
    bb.setPosition(&p);
    bb.setDimensions(&d);
    pObject->setBoundingBox(&bb);
    // we create a style for the object
    pStyle=pRenderInfo->createStyle("style_for_"+os.str());
    pStyle->addId(pObject->getId());
    pGroup=pStyle->getGroup();
    pGroup->setFillColor("linear_gradient_pad_narrow");
    pGroup->setStroke("#000000");
    pGroup->setStrokeWidth(2.0);
    // add an rectangle to the group
    pRectangle=pGroup->createRectangle();
    pRectangle->setCoordinates(RelAbsVector(0.0,0.0),RelAbsVector(0.0,0.0),RelAbsVector(0.0,0.0));
    pRectangle->setSize(RelAbsVector(0.0,100),RelAbsVector(0.0,100));
    y+=1.2*HEIGHT;

    // eighth rectangle
    // create a graphical object with the correct bounding box
    os.str("");
    os << "graphical_object_" << object_index++;
    pObject=pLayout->createAdditionalGraphicalObject();
    pObject->setId(os.str());
    // we set the bounding box
    p=Point(x,y);
    bb.setPosition(&p);
    bb.setDimensions(&d);
    pObject->setBoundingBox(&bb);
    // we create a style for the object
    pStyle=pRenderInfo->createStyle("style_for_"+os.str());
    pStyle->addId(pObject->getId());
    pGroup=pStyle->getGroup();
    pGroup->setFillColor("linear_gradient_reflect_narrow");
    pGroup->setStroke("#000000");
    pGroup->setStrokeWidth(2.0);
    // add an rectangle to the group
    pRectangle=pGroup->createRectangle();
    pRectangle->setCoordinates(RelAbsVector(0.0,0.0),RelAbsVector(0.0,0.0),RelAbsVector(0.0,0.0));
    pRectangle->setSize(RelAbsVector(0.0,100),RelAbsVector(0.0,100));
    y+=1.2*HEIGHT;
    
    // nineth rectangle
    // create a graphical object with the correct bounding box
    os.str("");
    os << "graphical_object_" << object_index++;
    pObject=pLayout->createAdditionalGraphicalObject();
    pObject->setId(os.str());
    // we set the bounding box
    p=Point(x,y);
    bb.setPosition(&p);
    bb.setDimensions(&d);
    pObject->setBoundingBox(&bb);
    // we create a style for the object
    pStyle=pRenderInfo->createStyle("style_for_"+os.str());
    pStyle->addId(pObject->getId());
    pGroup=pStyle->getGroup();
    pGroup->setFillColor("linear_gradient_repeat_narrow");
    pGroup->setStroke("#000000");
    pGroup->setStrokeWidth(2.0);
    // add an rectangle to the group
    pRectangle=pGroup->createRectangle();
    pRectangle->setCoordinates(RelAbsVector(0.0,0.0),RelAbsVector(0.0,0.0),RelAbsVector(0.0,0.0));
    pRectangle->setSize(RelAbsVector(0.0,100),RelAbsVector(0.0,100));
    y+=1.2*HEIGHT;

    // tenth rectangle
    // create a graphical object with the correct bounding box
    os.str("");
    os << "graphical_object_" << object_index++;
    pObject=pLayout->createAdditionalGraphicalObject();
    pObject->setId(os.str());
    // we set the bounding box
    p=Point(x,y);
    bb.setPosition(&p);
    bb.setDimensions(&d);
    pObject->setBoundingBox(&bb);
    // we create a style for the object
    pStyle=pRenderInfo->createStyle("style_for_"+os.str());
    pStyle->addId(pObject->getId());
    pGroup=pStyle->getGroup();
    pGroup->setFillColor("linear_gradient_diag_pad_narrow");
    pGroup->setStroke("#000000");
    pGroup->setStrokeWidth(2.0);
    // add an rectangle to the group
    pRectangle=pGroup->createRectangle();
    pRectangle->setCoordinates(RelAbsVector(0.0,0.0),RelAbsVector(0.0,0.0),RelAbsVector(0.0,0.0));
    pRectangle->setSize(RelAbsVector(0.0,100),RelAbsVector(0.0,100));
    y+=1.2*HEIGHT;
    
    // eleventh rectangle
    // create a graphical object with the correct bounding box
    os.str("");
    os << "graphical_object_" << object_index++;
    pObject=pLayout->createAdditionalGraphicalObject();
    pObject->setId(os.str());
    // we set the bounding box
    p=Point(x,y);
    bb.setPosition(&p);
    bb.setDimensions(&d);
    pObject->setBoundingBox(&bb);
    // we create a style for the object
    pStyle=pRenderInfo->createStyle("style_for_"+os.str());
    pStyle->addId(pObject->getId());
    pGroup=pStyle->getGroup();
    pGroup->setFillColor("linear_gradient_diag_reflect_narrow");
    pGroup->setStroke("#000000");
    pGroup->setStrokeWidth(2.0);
    // add an rectangle to the group
    pRectangle=pGroup->createRectangle();
    pRectangle->setCoordinates(RelAbsVector(0.0,0.0),RelAbsVector(0.0,0.0),RelAbsVector(0.0,0.0));
    pRectangle->setSize(RelAbsVector(0.0,100),RelAbsVector(0.0,100));
    y+=1.2*HEIGHT;
    
    // twelfth rectangle
    // create a graphical object with the correct bounding box
    os.str("");
    os << "graphical_object_" << object_index++;
    pObject=pLayout->createAdditionalGraphicalObject();
    pObject->setId(os.str());
    // we set the bounding box
    p=Point(x,y);
    bb.setPosition(&p);
    bb.setDimensions(&d);
    pObject->setBoundingBox(&bb);
    // we create a style for the object
    pStyle=pRenderInfo->createStyle("style_for_"+os.str());
    pStyle->addId(pObject->getId());
    pGroup=pStyle->getGroup();
    pGroup->setFillColor("linear_gradient_diag_repeat_narrow");
    pGroup->setStroke("#000000");
    pGroup->setStrokeWidth(2.0);
    // add an rectangle to the group
    pRectangle=pGroup->createRectangle();
    pRectangle->setCoordinates(RelAbsVector(0.0,0.0),RelAbsVector(0.0,0.0),RelAbsVector(0.0,0.0));
    pRectangle->setSize(RelAbsVector(0.0,100),RelAbsVector(0.0,100));

    // write the document
    SBMLWriter writer = SBMLWriter();
    std::ostringstream fos;
    fos << "linear_gradient_two_stops_l" << level << "v" << version << ".xml";
    writer.writeSBML(pDocument,fos.str());
    delete pDocument;
}
