/* Copyright 2010 Ralph Gauges
 *
 * This library is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation.  A copy of the license agreement is
 * provided in the file named "LICENSE.txt" included with this software
 * distribution.  It is also available online at
 * http://www.gnu.org/licenses/lgpl.html
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with this library; if not, write to the Free Software Foundation,
 * Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA.
 *
 * The original code contained here was initially developed by:
 *
 *     Ralph Gauges
 *     Group for the modeling of biological processes 
 *     University of Heidelberg
 *     Im Neuenheimer Feld 267
 *     69120 Heidelberg
 *     Germany
 *
 *     mailto:ralph.gauges@bioquant.uni-heidelberg.de
 *
 * Contributor(s):
 *
 */

#include <sbml/SBMLDocument.h>
#include <sbml/SBMLWriter.h>
#include <sbml/Model.h>
#include <sbml/layout/Layout.h>
#include <sbml/layout/BoundingBox.h>
#include <sbml/layout/Point.h>
#include <sbml/layout/Dimensions.h>
#include <sbml/layout/GraphicalObject.h>
#include <sbml/layout/render/LocalRenderInformation.h>
#include <sbml/layout/render/GlobalRenderInformation.h>
#include <sbml/layout/render/LocalStyle.h>
#include <sbml/layout/render/GlobalStyle.h>
#include <sbml/layout/render/RenderPoint.h>
#include <sbml/layout/render/RenderCurve.h>
#include <sbml/layout/render/LineEnding.h>
#include <sbml/layout/render/Group.h>

#include <cmath>
#include <sstream>
#include <string>
#include <stdlib.h>
#include <iostream>

/**
 * This example draw a series of straight lines with a simple line ending on
 * both end. The lineending is defined twice, once with rotational mapping
 * enabled and once without rotational mapping
 */


using namespace std;

int main(int argc,char** argv)
{
    // first we create a new SBML Document with a model
    unsigned int level=SBML_DEFAULT_LEVEL;
    unsigned int version=SBML_DEFAULT_VERSION;
    SBMLDocument* pDocument=NULL;
    if(argc != 1 && argc != 3)
    {
        std::cout << "usage: " << argv[0] << " [LEVEL VERSION]" << std::endl;
        std::cout << "e.g. " << argv[0] << " 2 3" << std::endl;
        std::cout << "or " << argv[0] << std::endl;
        return 0;
    }
    else
    {
       if(argc == 1)
       {
           pDocument=new SBMLDocument(level,version);
       }
       else
       {
         level=strtol(argv[1],NULL,10); 
         version=strtol(argv[2],NULL,10); 
         try{
           pDocument=new SBMLDocument(level,version);
           if(!pDocument->hasValidLevelVersionNamespaceCombination())
           {
             std::cerr << "Invalid Level and/or version. Aborting." << std::endl;
             return 1;
           }
         }
         catch(...)
         {
            std::cerr << "Invalid Level and/or version. Aborting." << std::endl;
            return 1;
         }
       }

    }
    assert(pDocument != NULL);
    Model* pModel=pDocument->createModel("Model_1");
    assert(pModel != NULL);
    if(level > 2)
    {
       UnitDefinition* pUDef=pModel->createUnitDefinition();
       pUDef->setId("squaremetre");
       pUDef->setName("squaremetre");
       Unit* pUnit=pUDef->createUnit();
       pUnit->setKind(UNIT_KIND_METRE);
       pUnit->setMultiplier(1.0);
       pUnit->setScale(0);
       pUnit->setExponent(2.0);
       pModel->setTimeUnits("second"); 
       pModel->setSubstanceUnits("mole"); 
       pModel->setVolumeUnits("litre"); 
       pModel->setAreaUnits("squaremetre"); 
       pModel->setLengthUnits("metre"); 
       pModel->setExtentUnits("mole"); 
    }

    // now we create a new layout
    Layout* pLayout=pModel->createLayout();
    pLayout->setId("LineendingsTest");
    // define the number of segments to create for each line width
    const unsigned int NUM_SEGMENTS=12; 
    unsigned int i;
    double y=5.0,x=5.0;
    // draw each line in a 70x70 box
    const double WIDTH=75.0;
    double HEIGHT=75.0;
    double angle=0.0;
    const double PHI=2*M_PI/NUM_SEGMENTS;
    ostringstream os;
    unsigned int object_index=1;
    // the dimensions of the whole layout will be
    Dimensions dim(NUM_SEGMENTS*WIDTH*1.2+10,2*HEIGHT*1.2+10);
    pLayout->setDimensions(&dim);
    // create the render information
    LocalRenderInformation* pRenderInfo=pLayout->createLocalRenderInformation();
    pRenderInfo->setId("LineendingsRenderInfo");
    // define the two line endings
    LineEnding* pLE=pRenderInfo->createLineEnding();
    pLE->setId("unrotated_ending");
    pLE->setEnableRotationalMapping(false);
    BoundingBox bb;
    Point p(-10.0,-10.0);
    Dimensions d(10.0,20.0);
    bb.setPosition(&p);
    bb.setDimensions(&d);
    pLE->setBoundingBox(&bb);
    Group* pGroup=pLE->getGroup();
    pGroup->setStroke("#000000");
    pGroup->setStrokeWidth(2.0);
    // add two lines
    RenderCurve* pCurve=pGroup->createCurve();
    RenderPoint* pP=pCurve->createPoint();
    pP->setCoordinates(RelAbsVector(0.0,0.0),RelAbsVector(0.0,0.0),RelAbsVector(0.0,0.0));
    pP=pCurve->createPoint();
    pP->setCoordinates(RelAbsVector(0.0,100.0),RelAbsVector(0.0,50.0),RelAbsVector(0.0,0.0));
    pP=pCurve->createPoint();
    pP->setCoordinates(RelAbsVector(0.0,100.0),RelAbsVector(0.0,50.0),RelAbsVector(0.0,0.0));
    pP=pCurve->createPoint();
    pP->setCoordinates(RelAbsVector(0.0,0.0),RelAbsVector(0.0,100.0),RelAbsVector(0.0,0.0));
 
    // second line ending
    pLE=pRenderInfo->createLineEnding();
    pLE->setId("rotated_ending");
    pLE->setEnableRotationalMapping(true);
    bb.setPosition(&p);
    bb.setDimensions(&d);
    pLE->setBoundingBox(&bb);
    pGroup=pLE->getGroup();
    pGroup->setStroke("#000000");
    pGroup->setStrokeWidth(2.0);
    // add two lines
    pCurve=pGroup->createCurve();
    pP=pCurve->createPoint();
    pP->setCoordinates(RelAbsVector(0.0,0.0),RelAbsVector(0.0,0.0),RelAbsVector(0.0,0.0));
    pP=pCurve->createPoint();
    pP->setCoordinates(RelAbsVector(0.0,100.0),RelAbsVector(0.0,50.0),RelAbsVector(0.0,0.0));
    pP=pCurve->createPoint();
    pP->setCoordinates(RelAbsVector(0.0,100.0),RelAbsVector(0.0,50.0),RelAbsVector(0.0,0.0));
    pP=pCurve->createPoint();
    pP->setCoordinates(RelAbsVector(0.0,0.0),RelAbsVector(0.0,100.0),RelAbsVector(0.0,0.0));
 
    d=Dimensions(WIDTH,HEIGHT);


    for(i=0;i<NUM_SEGMENTS;++i)
    {
        angle=i*PHI;
        // scale it with 50%
        double dx=cos(angle)*50;
        y=5.0;
        // scale it with 50%
        double dy=sin(angle)*50;
        // create a graphical object with the correct bounding box
        os.str("");
        os << "graphical_object_" << object_index++;
        GraphicalObject* pObject=pLayout->createAdditionalGraphicalObject();
        pObject->setId(os.str());
        // we set the bound
        p=Point(x,y);
        bb.setPosition(&p);
        bb.setDimensions(&d);
        pObject->setBoundingBox(&bb);
        // we create a style for the object
        LocalStyle* pStyle=pRenderInfo->createStyle("style_for_"+os.str());
        pStyle->addId(pObject->getId());
        pGroup=pStyle->getGroup();
        pGroup->setStroke("#000000");
        pGroup->setStrokeWidth(2.0);
        pGroup->setStartHead("unrotated_ending");
        pGroup->setEndHead("unrotated_ending");
        // add a line segment to the group
        pCurve=pGroup->createCurve();
        pP=pCurve->createPoint();
        pP->setCoordinates(RelAbsVector(0.0,50.0-dx),RelAbsVector(0.0,50.0-dy));
        pP=pCurve->createPoint();
        pP->setCoordinates(RelAbsVector(0.0,50.0+dx),RelAbsVector(0.0,50.0+dy));
        y+=HEIGHT*1.2;
            // second line with rotation mapping
        // create a graphical object with the correct bounding box
        os.str("");
        os << "graphical_object_" << object_index++;
        pObject=pLayout->createAdditionalGraphicalObject();
        pObject->setId(os.str());
        // we set the bound
        p=Point(x,y);
        bb.setPosition(&p);
        bb.setDimensions(&d);
        pObject->setBoundingBox(&bb);
        // we create a style for the object
        pStyle=pRenderInfo->createStyle("style_for_"+os.str());
        pStyle->addId(pObject->getId());
        pGroup=pStyle->getGroup();
        pGroup->setStroke("#000000");
        pGroup->setStrokeWidth(2.0);
        pGroup->setStartHead("rotated_ending");
        pGroup->setEndHead("rotated_ending");
        // add a line segment to the group
        pCurve=pGroup->createCurve();
        pP=pCurve->createPoint();
        pP->setCoordinates(RelAbsVector(0.0,50.0-dx),RelAbsVector(0.0,50.0-dy));
        pP=pCurve->createPoint();
        pP->setCoordinates(RelAbsVector(0.0,50.0+dx),RelAbsVector(0.0,50.0+dy));
        x+=WIDTH*1.2;
    }
    // write the document
    SBMLWriter writer = SBMLWriter();
    std::ostringstream fos;
    fos << "lineendings_straightlines_l" << level << "v" << version << ".xml";
    writer.writeSBML(pDocument,fos.str());
    delete pDocument;
}
