/* Copyright 2010 Ralph Gauges
 *
 * This library is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation.  A copy of the license agreement is
 * provided in the file named "LICENSE.txt" included with this software
 * distribution.  It is also available online at
 * http://www.gnu.org/licenses/lgpl.html
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with this library; if not, write to the Free Software Foundation,
 * Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA.
 *
 * The original code contained here was initially developed by:
 *
 *     Ralph Gauges
 *     Group for the modeling of biological processes 
 *     University of Heidelberg
 *     Im Neuenheimer Feld 267
 *     69120 Heidelberg
 *     Germany
 *
 *     mailto:ralph.gauges@bioquant.uni-heidelberg.de
 *
 * Contributor(s):
 *
 */

#include <sbml/SBMLDocument.h>
#include <sbml/SBMLWriter.h>
#include <sbml/Model.h>
#include <sbml/layout/Layout.h>
#include <sbml/layout/BoundingBox.h>
#include <sbml/layout/Point.h>
#include <sbml/layout/Dimensions.h>
#include <sbml/layout/GraphicalObject.h>
#include <sbml/layout/render/LocalRenderInformation.h>
#include <sbml/layout/render/GlobalRenderInformation.h>
#include <sbml/layout/render/LocalStyle.h>
#include <sbml/layout/render/GlobalStyle.h>
#include <sbml/layout/render/RenderPoint.h>
#include <sbml/layout/render/RenderCurve.h>

#include <cmath>
#include <sstream>
#include <string>
#include <vector>
#include <stdlib.h>
#include <iostream>

using namespace std;

/**
 * This test file creates a set of horizontal lines with different line
 * stippling patterns.
 */

int main(int argc,char** argv)
{
    // first we create a new SBML Document with a model
    unsigned int level=SBML_DEFAULT_LEVEL;
    unsigned int version=SBML_DEFAULT_VERSION;
    SBMLDocument* pDocument=NULL;
    if(argc != 1 && argc != 3)
    {
        std::cout << "usage: " << argv[0] << " [LEVEL VERSION]" << std::endl;
        std::cout << "e.g. " << argv[0] << " 2 3" << std::endl;
        std::cout << "or " << argv[0] << std::endl;
        return 0;
    }
    else
    {
       if(argc == 1)
       {
           pDocument=new SBMLDocument(level,version);
       }
       else
       {
         level=strtol(argv[1],NULL,10); 
         version=strtol(argv[2],NULL,10); 
         try{
           pDocument=new SBMLDocument(level,version);
           if(!pDocument->hasValidLevelVersionNamespaceCombination())
           {
             std::cerr << "Invalid Level and/or version. Aborting." << std::endl;
             return 1;
           }
         }
         catch(...)
         {
            std::cerr << "Invalid Level and/or version. Aborting." << std::endl;
            return 1;
         }
       }

    }
    assert(pDocument != NULL);
    Model* pModel=pDocument->createModel("Model_1");
    assert(pModel != NULL);
    if(level > 2)
    {
       UnitDefinition* pUDef=pModel->createUnitDefinition();
       pUDef->setId("squaremetre");
       pUDef->setName("squaremetre");
       Unit* pUnit=pUDef->createUnit();
       pUnit->setKind(UNIT_KIND_METRE);
       pUnit->setMultiplier(1.0);
       pUnit->setScale(0);
       pUnit->setExponent(2.0);
       pModel->setTimeUnits("second"); 
       pModel->setSubstanceUnits("mole"); 
       pModel->setVolumeUnits("litre"); 
       pModel->setAreaUnits("squaremetre"); 
       pModel->setLengthUnits("metre"); 
       pModel->setExtentUnits("mole"); 
    }

    // now we create a new layout
    Layout* pLayout=pModel->createLayout();
    pLayout->setId("LineTest");
    // define a few line widths to test
    const unsigned int NUM_LINES=6;
    double y=5.0,x=5.0;
    // draw each line in a 300x30 box
    const double WIDTH=300.0;
    double HEIGHT=30.0;
    ostringstream os;
    unsigned int object_index=1;
    // the dimensions of the whole layout will be
    Dimensions dim(WIDTH+10,NUM_LINES*HEIGHT*1.2+10);
    pLayout->setDimensions(&dim);
    // create the render information
    LocalRenderInformation* pRenderInfo=pLayout->createLocalRenderInformation();
    pRenderInfo->setId("LineRenderInfo");
    // we have a black background with green lines
    pRenderInfo->setBackgroundColor("#000000");
    Dimensions d(WIDTH,HEIGHT);
    std::vector<unsigned int> v;

    GraphicalObject* pObject=NULL;
    BoundingBox bb;
    Point p;
    LocalStyle* pStyle=NULL;
    Group* pGroup=NULL;
    RenderCurve* pCurve=NULL;
    RenderPoint* pP=NULL;

    // first line
    // create a graphical object with the correct bounding box
    os.str("");
    os << "graphical_object_" << object_index++;
    pObject=pLayout->createAdditionalGraphicalObject();
    pObject->setId(os.str());
    // we set the bound
    p=Point(x,y);
    bb.setPosition(&p);
    bb.setDimensions(&d);
    pObject->setBoundingBox(&bb);
    // we create a style for the object
    pStyle=pRenderInfo->createStyle("style_for_"+os.str());
    pStyle->addId(pObject->getId());
    pGroup=pStyle->getGroup();
    pGroup->setStroke("#00FF00");
    pGroup->setStrokeWidth(3.0);
    v.clear();
    v.push_back(32);
    v.push_back(32);
    pGroup->setDashArray(v);
    // add a line segment to the group
    pCurve=pGroup->createCurve();
    pP=pCurve->createPoint();
    pP->setCoordinates(RelAbsVector(0.0,10.0),RelAbsVector(0.0,50.0));
    pP=pCurve->createPoint();
    pP->setCoordinates(RelAbsVector(0.0,90.0),RelAbsVector(0.0,50.0));
    y+=HEIGHT*1.2;
    
    // second line
    os.str("");
    os << "graphical_object_" << object_index++;
    pObject=pLayout->createAdditionalGraphicalObject();
    pObject->setId(os.str());
    // we set the bound
    p=Point(x,y);
    bb.setPosition(&p);
    bb.setDimensions(&d);
    pObject->setBoundingBox(&bb);
    // we create a style for the object
    pStyle=pRenderInfo->createStyle("style_for_"+os.str());
    pStyle->addId(pObject->getId());
    pGroup=pStyle->getGroup();
    pGroup->setStroke("#00FF00");
    pGroup->setStrokeWidth(3.0);
    v.clear();
    v.push_back(20);
    v.push_back(10);
    pGroup->setDashArray(v);
    // add a line segment to the group
    pCurve=pGroup->createCurve();
    pP=pCurve->createPoint();
    pP->setCoordinates(RelAbsVector(0.0,10.0),RelAbsVector(0.0,50.0));
    pP=pCurve->createPoint();
    pP->setCoordinates(RelAbsVector(0.0,90.0),RelAbsVector(0.0,50.0));
    y+=HEIGHT*1.2;

    // third line
    os.str("");
    os << "graphical_object_" << object_index++;
    pObject=pLayout->createAdditionalGraphicalObject();
    pObject->setId(os.str());
    // we set the bound
    p=Point(x,y);
    bb.setPosition(&p);
    bb.setDimensions(&d);
    pObject->setBoundingBox(&bb);
    // we create a style for the object
    pStyle=pRenderInfo->createStyle("style_for_"+os.str());
    pStyle->addId(pObject->getId());
    pGroup=pStyle->getGroup();
    pGroup->setStroke("#00FF00");
    pGroup->setStrokeWidth(3.0);
    v.clear();
    v.push_back(10);
    v.push_back(20);
    pGroup->setDashArray(v);
    // add a line segment to the group
    pCurve=pGroup->createCurve();
    pP=pCurve->createPoint();
    pP->setCoordinates(RelAbsVector(0.0,10.0),RelAbsVector(0.0,50.0));
    pP=pCurve->createPoint();
    pP->setCoordinates(RelAbsVector(0.0,90.0),RelAbsVector(0.0,50.0));
    y+=HEIGHT*1.2;

    // fourth line
    os.str("");
    os << "graphical_object_" << object_index++;
    pObject=pLayout->createAdditionalGraphicalObject();
    pObject->setId(os.str());
    // we set the bound
    p=Point(x,y);
    bb.setPosition(&p);
    bb.setDimensions(&d);
    pObject->setBoundingBox(&bb);
    // we create a style for the object
    pStyle=pRenderInfo->createStyle("style_for_"+os.str());
    pStyle->addId(pObject->getId());
    pGroup=pStyle->getGroup();
    pGroup->setStroke("#00FF00");
    pGroup->setStrokeWidth(3.0);
    // add a line segment to the group
    pCurve=pGroup->createCurve();
    pP=pCurve->createPoint();
    pP->setCoordinates(RelAbsVector(0.0,10.0),RelAbsVector(0.0,50.0));
    pP=pCurve->createPoint();
    pP->setCoordinates(RelAbsVector(0.0,90.0),RelAbsVector(0.0,50.0));
    y+=HEIGHT*1.2;

    // fifth line
    os.str("");
    os << "graphical_object_" << object_index++;
    pObject=pLayout->createAdditionalGraphicalObject();
    pObject->setId(os.str());
    // we set the bound
    p=Point(x,y);
    bb.setPosition(&p);
    bb.setDimensions(&d);
    pObject->setBoundingBox(&bb);
    // we create a style for the object
    pStyle=pRenderInfo->createStyle("style_for_"+os.str());
    pStyle->addId(pObject->getId());
    pGroup=pStyle->getGroup();
    pGroup->setStroke("#00FF00");
    pGroup->setStrokeWidth(3.0);
    v.clear();
    v.push_back(20);
    v.push_back(20);
    pGroup->setDashArray(v);
    // add a line segment to the group
    pCurve=pGroup->createCurve();
    pP=pCurve->createPoint();
    pP->setCoordinates(RelAbsVector(0.0,10.0),RelAbsVector(0.0,50.0));
    pP=pCurve->createPoint();
    pP->setCoordinates(RelAbsVector(0.0,90.0),RelAbsVector(0.0,50.0));
    y+=HEIGHT*1.2;

    // sixth line
    os.str("");
    os << "graphical_object_" << object_index++;
    pObject=pLayout->createAdditionalGraphicalObject();
    pObject->setId(os.str());
    // we set the bound
    p=Point(x,y);
    bb.setPosition(&p);
    bb.setDimensions(&d);
    pObject->setBoundingBox(&bb);
    // we create a style for the object
    pStyle=pRenderInfo->createStyle("style_for_"+os.str());
    pStyle->addId(pObject->getId());
    pGroup=pStyle->getGroup();
    pGroup->setStroke("#00FF00");
    pGroup->setStrokeWidth(3.0);
    v.clear();
    v.push_back(10);
    v.push_back(5);
    v.push_back(5);
    v.push_back(10);
    v.push_back(20);
    v.push_back(10);
    pGroup->setDashArray(v);
    // add a line segment to the group
    pCurve=pGroup->createCurve();
    pP=pCurve->createPoint();
    pP->setCoordinates(RelAbsVector(0.0,10.0),RelAbsVector(0.0,50.0));
    pP=pCurve->createPoint();
    pP->setCoordinates(RelAbsVector(0.0,90.0),RelAbsVector(0.0,50.0));
    y+=HEIGHT*1.2;

    // seventh line
    os.str("");
    os << "graphical_object_" << object_index++;
    pObject=pLayout->createAdditionalGraphicalObject();
    pObject->setId(os.str());
    // we set the bound
    p=Point(x,y);
    bb.setPosition(&p);
    bb.setDimensions(&d);
    pObject->setBoundingBox(&bb);
    // we create a style for the object
    pStyle=pRenderInfo->createStyle("style_for_"+os.str());
    pStyle->addId(pObject->getId());
    pGroup=pStyle->getGroup();
    pGroup->setStroke("#00FF00");
    pGroup->setStrokeWidth(3.0);
    v.clear();
    v.push_back(0);
    v.push_back(50);
    v.push_back(20);
    pGroup->setDashArray(v);
    // add a line segment to the group
    pCurve=pGroup->createCurve();
    pP=pCurve->createPoint();
    pP->setCoordinates(RelAbsVector(0.0,10.0),RelAbsVector(0.0,50.0));
    pP=pCurve->createPoint();
    pP->setCoordinates(RelAbsVector(0.0,90.0),RelAbsVector(0.0,50.0));
    y+=HEIGHT*1.2;

    // eight line
    os.str("");
    os << "graphical_object_" << object_index++;
    pObject=pLayout->createAdditionalGraphicalObject();
    pObject->setId(os.str());
    // we set the bound
    p=Point(x,y);
    bb.setPosition(&p);
    bb.setDimensions(&d);
    pObject->setBoundingBox(&bb);
    // we create a style for the object
    pStyle=pRenderInfo->createStyle("style_for_"+os.str());
    pStyle->addId(pObject->getId());
    pGroup=pStyle->getGroup();
    pGroup->setStroke("#00FF00");
    pGroup->setStrokeWidth(3.0);
    // add a line segment to the group
    pCurve=pGroup->createCurve();
    pP=pCurve->createPoint();
    pP->setCoordinates(RelAbsVector(0.0,10.0),RelAbsVector(0.0,50.0));
    pP=pCurve->createPoint();
    pP->setCoordinates(RelAbsVector(0.0,90.0),RelAbsVector(0.0,50.0));
    y+=HEIGHT*1.2;

    // write the document
    SBMLWriter writer = SBMLWriter();
    std::ostringstream fos;
    fos << "linestipple_horizontal_line_l" << level << "v" << version << ".xml";
    writer.writeSBML(pDocument,fos.str());
    delete pDocument;
}
