/* Copyright 2010 Ralph Gauges
 *
 * This library is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation.  A copy of the license agreement is
 * provided in the file named "LICENSE.txt" included with this software
 * distribution.  It is also available online at
 * http://www.gnu.org/licenses/lgpl.html
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with this library; if not, write to the Free Software Foundation,
 * Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA.
 *
 * The original code contained here was initially developed by:
 *
 *     Ralph Gauges
 *     Group for the modeling of biological processes 
 *     University of Heidelberg
 *     Im Neuenheimer Feld 267
 *     69120 Heidelberg
 *     Germany
 *
 *     mailto:ralph.gauges@bioquant.uni-heidelberg.de
 *
 * Contributor(s):
 *
 */

#include <sbml/SBMLDocument.h>
#include <sbml/SBMLWriter.h>
#include <sbml/Model.h>
#include <sbml/layout/Layout.h>
#include <sbml/layout/BoundingBox.h>
#include <sbml/layout/Point.h>
#include <sbml/layout/Dimensions.h>
#include <sbml/layout/GraphicalObject.h>
#include <sbml/layout/render/LocalRenderInformation.h>
#include <sbml/layout/render/GlobalRenderInformation.h>
#include <sbml/layout/render/LocalStyle.h>
#include <sbml/layout/render/GlobalStyle.h>
#include <sbml/layout/render/Rectangle.h>

#include <cmath>
#include <sstream>
#include <string>
#include <stdlib.h>
#include <iostream>

using namespace std;

/**
 * This example draws three rectangles.
 * One with both sides having the same length, one
 * where the width is larger and one where the height is larger.
 * All three rectangles are filled with a single color and have an outline.
 */

int main(int argc,char** argv)
{
    // first we create a new SBML Document with a model
    unsigned int level=SBML_DEFAULT_LEVEL;
    unsigned int version=SBML_DEFAULT_VERSION;
    SBMLDocument* pDocument=NULL;
    if(argc != 1 && argc != 3)
    {
        std::cout << "usage: " << argv[0] << " [LEVEL VERSION]" << std::endl;
        std::cout << "e.g. " << argv[0] << " 2 3" << std::endl;
        std::cout << "or " << argv[0] << std::endl;
        return 0;
    }
    else
    {
       if(argc == 1)
       {
           pDocument=new SBMLDocument(level,version);
       }
       else
       {
         level=strtol(argv[1],NULL,10); 
         version=strtol(argv[2],NULL,10); 
         try{
           pDocument=new SBMLDocument(level,version);
           if(!pDocument->hasValidLevelVersionNamespaceCombination())
           {
             std::cerr << "Invalid Level and/or version. Aborting." << std::endl;
             return 1;
           }
         }
         catch(...)
         {
            std::cerr << "Invalid Level and/or version. Aborting." << std::endl;
            return 1;
         }
       }

    }
    assert(pDocument != NULL);
    Model* pModel=pDocument->createModel("Model_1");
    assert(pModel != NULL);
    if(level > 2)
    {
       UnitDefinition* pUDef=pModel->createUnitDefinition();
       pUDef->setId("squaremetre");
       pUDef->setName("squaremetre");
       Unit* pUnit=pUDef->createUnit();
       pUnit->setKind(UNIT_KIND_METRE);
       pUnit->setMultiplier(1.0);
       pUnit->setScale(0);
       pUnit->setExponent(2.0);
       pModel->setTimeUnits("second"); 
       pModel->setSubstanceUnits("mole"); 
       pModel->setVolumeUnits("litre"); 
       pModel->setAreaUnits("squaremetre"); 
       pModel->setLengthUnits("metre"); 
       pModel->setExtentUnits("mole"); 
    }

    // now we create a new layout
    Layout* pLayout=pModel->createLayout();
    pLayout->setId("RectangleTest");
    const double MARGIN_X=5.0;
    const double MARGIN_Y=5.0;
    const double H_SPACE=50.0;
    const double V_SPACE=50.0;
    // draw each line in a 30x30 box
    const double SIDE_LARGE=200;
    const double SIDE_SMALL=100;
    ostringstream os;
    unsigned int object_index=1;
    // the dimensions of the whole layout will be
    Dimensions dim(2*MARGIN_X+SIDE_LARGE+H_SPACE+SIDE_SMALL,2*MARGIN_Y+2*SIDE_SMALL+V_SPACE);
    pLayout->setDimensions(&dim);
    // create the render information
    LocalRenderInformation* pRenderInfo=pLayout->createLocalRenderInformation();
    pRenderInfo->setId("RectangleRenderInfo");
    
    // first rectangle
    // create a graphical object with the correct bounding box
    os.str("");
    os << "graphical_object_" << object_index++;
    GraphicalObject* pObject=pLayout->createAdditionalGraphicalObject();
    pObject->setId(os.str());
    // we set the bounding box
    BoundingBox bb;
    Dimensions d(SIDE_LARGE,SIDE_SMALL);
    Point p(MARGIN_X,MARGIN_Y);
    bb.setPosition(&p);
    bb.setDimensions(&d);
    pObject->setBoundingBox(&bb);
    // we create a style for the object
    LocalStyle* pStyle=pRenderInfo->createStyle("style_for_"+os.str());
    pStyle->addId(pObject->getId());
    Group* pGroup=pStyle->getGroup();
    pGroup->setStroke("#000000");
    pGroup->setStrokeWidth(3.0);
    pGroup->setFillColor("#FF0000");
    // add an rectangle to the group
    Rectangle* pRectangle=pGroup->createRectangle();
    pRectangle->setCoordinates(RelAbsVector(0.0,0.0),RelAbsVector(0.0,0.0),RelAbsVector(0.0,0.0));
    pRectangle->setSize(RelAbsVector(0.0,100),RelAbsVector(0.0,100));
    
    // second rectangle
    // create a graphical object with the correct bounding box
    os.str("");
    os << "graphical_object_" << object_index++;
    pObject=pLayout->createAdditionalGraphicalObject();
    pObject->setId(os.str());
    // we set the bounding box
    d=Dimensions(SIDE_SMALL,SIDE_LARGE);
    p=Point(MARGIN_X+SIDE_LARGE+H_SPACE,MARGIN_Y);
    bb.setPosition(&p);
    bb.setDimensions(&d);
    pObject->setBoundingBox(&bb);
    // we create a style for the object
    pStyle=pRenderInfo->createStyle("style_for_"+os.str());
    pStyle->addId(pObject->getId());
    pGroup=pStyle->getGroup();
    pGroup->setStroke("#000000");
    pGroup->setStrokeWidth(3.0);
    pGroup->setFillColor("#FF0000");
    // add a rectangle to the group
    pRectangle=pGroup->createRectangle();
    pRectangle->setCoordinates(RelAbsVector(0.0,0.0),RelAbsVector(0.0,0.0),RelAbsVector(0.0,0.0));
    pRectangle->setSize(RelAbsVector(0.0,100),RelAbsVector(0.0,100));
    
    // third rectangle
    // create a graphical object with the correct bounding box
    os.str("");
    os << "graphical_object_" << object_index++;
    pObject=pLayout->createAdditionalGraphicalObject();
    pObject->setId(os.str());
    // we set the bounding box
    d=Dimensions(SIDE_SMALL,SIDE_SMALL);
    p=Point(MARGIN_X+(SIDE_LARGE-SIDE_SMALL)/2.0,MARGIN_Y+SIDE_SMALL+V_SPACE);
    bb.setPosition(&p);
    bb.setDimensions(&d);
    pObject->setBoundingBox(&bb);
    // we create a style for the object
    pStyle=pRenderInfo->createStyle("style_for_"+os.str());
    pStyle->addId(pObject->getId());
    pGroup=pStyle->getGroup();
    pGroup->setStroke("#000000");
    pGroup->setStrokeWidth(3.0);
    pGroup->setFillColor("#FF0000");
    // add an rectangle to the group
    pRectangle=pGroup->createRectangle();
    pRectangle->setCoordinates(RelAbsVector(0.0,0.0),RelAbsVector(0.0,0.0),RelAbsVector(0.0,0.0));
    pRectangle->setSize(RelAbsVector(0.0,100),RelAbsVector(0.0,100));
    // write the document
    SBMLWriter writer = SBMLWriter();
    std::ostringstream fos;
    fos << "rectangle_mono_filled_outline_l" << level << "v" << version << ".xml";
    writer.writeSBML(pDocument,fos.str());
    delete pDocument;
}
