/* Copyright 2010 Ralph Gauges
 *
 * This library is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation.  A copy of the license agreement is
 * provided in the file named "LICENSE.txt" included with this software
 * distribution.  It is also available online at
 * http://www.gnu.org/licenses/lgpl.html
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with this library; if not, write to the Free Software Foundation,
 * Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA.
 *
 * The original code contained here was initially developed by:
 *
 *     Ralph Gauges
 *     Group for the modeling of biological processes 
 *     University of Heidelberg
 *     Im Neuenheimer Feld 267
 *     69120 Heidelberg
 *     Germany
 *
 *     mailto:ralph.gauges@bioquant.uni-heidelberg.de
 *
 * Contributor(s):
 *
 */

#include <sbml/SBMLDocument.h>
#include <sbml/SBMLWriter.h>
#include <sbml/Model.h>
#include <sbml/layout/Layout.h>
#include <sbml/layout/BoundingBox.h>
#include <sbml/layout/Point.h>
#include <sbml/layout/Dimensions.h>
#include <sbml/layout/GraphicalObject.h>
#include <sbml/layout/render/LocalRenderInformation.h>
#include <sbml/layout/render/GlobalRenderInformation.h>
#include <sbml/layout/render/LocalStyle.h>
#include <sbml/layout/render/GlobalStyle.h>
#include <sbml/layout/render/RelAbsVector.h>
#include <sbml/layout/render/RenderPoint.h>
#include <sbml/layout/render/Rectangle.h>
#include <sbml/layout/render/Text.h>

#include <cmath>
#include <sstream>
#include <string>
#include <stdlib.h>
#include <iostream>

/**
 * This example draws a series strings with different vertical placement.
 */

using namespace std;

int main(int argc,char** argv)
{
    // first we create a new SBML Document with a model
    unsigned int level=SBML_DEFAULT_LEVEL;
    unsigned int version=SBML_DEFAULT_VERSION;
    SBMLDocument* pDocument=NULL;
    if(argc != 1 && argc != 3)
    {
        std::cout << "usage: " << argv[0] << " [LEVEL VERSION]" << std::endl;
        std::cout << "e.g. " << argv[0] << " 2 3" << std::endl;
        std::cout << "or " << argv[0] << std::endl;
        return 0;
    }
    else
    {
       if(argc == 1)
       {
           pDocument=new SBMLDocument(level,version);
       }
       else
       {
         level=strtol(argv[1],NULL,10); 
         version=strtol(argv[2],NULL,10); 
         try{
           pDocument=new SBMLDocument(level,version);
           if(!pDocument->hasValidLevelVersionNamespaceCombination())
           {
             std::cerr << "Invalid Level and/or version. Aborting." << std::endl;
             return 1;
           }
         }
         catch(...)
         {
            std::cerr << "Invalid Level and/or version. Aborting." << std::endl;
            return 1;
         }
       }

    }
    assert(pDocument != NULL);
    Model* pModel=pDocument->createModel("Model_1");
    assert(pModel != NULL);
    if(level > 2)
    {
       UnitDefinition* pUDef=pModel->createUnitDefinition();
       pUDef->setId("squaremetre");
       pUDef->setName("squaremetre");
       Unit* pUnit=pUDef->createUnit();
       pUnit->setKind(UNIT_KIND_METRE);
       pUnit->setMultiplier(1.0);
       pUnit->setScale(0);
       pUnit->setExponent(2.0);
       pModel->setTimeUnits("second"); 
       pModel->setSubstanceUnits("mole"); 
       pModel->setVolumeUnits("litre"); 
       pModel->setAreaUnits("squaremetre"); 
       pModel->setLengthUnits("metre"); 
       pModel->setExtentUnits("mole"); 
    }

    // now we create a new layout
    Layout* pLayout=pModel->createLayout();
    pLayout->setId("TextTest");
    double y=5.0,x=5.0;
    // draw each text in a 500x40 box
    const double WIDTH=500.0;
    double HEIGHT=40.0;
    ostringstream os;
    unsigned int object_index=1;
    const std::string text="abcdefghijklmnopqrstuvwxyz";
    // the dimensions of the whole layout will be
    Dimensions dim(WIDTH+10,9*HEIGHT*1.2+10);
    pLayout->setDimensions(&dim);
    // create the render information
    LocalRenderInformation* pRenderInfo=pLayout->createLocalRenderInformation();
    pRenderInfo->setId("TextRenderInfo");
    Dimensions d(WIDTH,HEIGHT);
    // create a graphical object with the correct bounding box
    // left alligned capital letters
    os.str("");
    os << "graphical_object_" << object_index++;
    GraphicalObject* pObject=pLayout->createAdditionalGraphicalObject();
    pObject->setId(os.str());
    // we set the bound
    BoundingBox bb;
    Point p(x,y);
    bb.setPosition(&p);
    bb.setDimensions(&d);
    pObject->setBoundingBox(&bb);
    // we create a style for the object
    LocalStyle* pStyle=pRenderInfo->createStyle("style_for_"+os.str());
    pStyle->addId(pObject->getId());
    Group* pGroup=pStyle->getGroup();
    pGroup->setStroke("#000000");
    pGroup->setStrokeWidth(1.0);
    pGroup->setFontFamily("sans");
    pGroup->setFontSize(20.0);
    pGroup->setVTextAnchor(Text::ANCHOR_TOP);
    // add a rectangle to the group
    Rectangle* pRectangle=pGroup->createRectangle();
    pRectangle->setCoordinates(RelAbsVector(0.0,0.0),RelAbsVector(0.0,0.0),RelAbsVector(0.0,0.0));
    pRectangle->setSize(RelAbsVector(0.0,100.0),RelAbsVector(0.0,100.0));
    // add a text element to the group
    Text* pText=pGroup->createText();
    pText->setText(text);
    y+=HEIGHT*1.2;
    // create a graphical object with the correct bounding box
    // left alligned lowercase letters
    os.str("");
    os << "graphical_object_" << object_index++;
    pObject=pLayout->createAdditionalGraphicalObject();
    pObject->setId(os.str());
    // we set the bound
    p=Point(x,y);
    bb.setPosition(&p);
    bb.setDimensions(&d);
    pObject->setBoundingBox(&bb);
    // we create a style for the object
    pStyle=pRenderInfo->createStyle("style_for_"+os.str());
    pStyle->addId(pObject->getId());
    pGroup=pStyle->getGroup();
    pGroup->setStroke("#000000");
    pGroup->setStrokeWidth(1.0);
    pGroup->setFontFamily("sans");
    pGroup->setFontSize(20.0);
    pGroup->setVTextAnchor(Text::ANCHOR_MIDDLE);
    // add a rectangle to the group
    pRectangle=pGroup->createRectangle();
    pRectangle->setCoordinates(RelAbsVector(0.0,0.0),RelAbsVector(0.0,0.0),RelAbsVector(0.0,0.0));
    pRectangle->setSize(RelAbsVector(0.0,100.0),RelAbsVector(0.0,100.0));
    // add a text element to the group
    pText=pGroup->createText();
    pText->setText(text);
    y+=HEIGHT*1.2;

    // create a graphical object with the correct bounding box
    // centered capital letters
    os.str("");
    os << "graphical_object_" << object_index++;
    pObject=pLayout->createAdditionalGraphicalObject();
    pObject->setId(os.str());
    // we set the bound
    p=Point(x,y);
    bb.setPosition(&p);
    bb.setDimensions(&d);
    pObject->setBoundingBox(&bb);
    // we create a style for the object
    pStyle=pRenderInfo->createStyle("style_for_"+os.str());
    pStyle->addId(pObject->getId());
    pGroup=pStyle->getGroup();
    pGroup->setStroke("#000000");
    pGroup->setStrokeWidth(1.0);
    pGroup->setFontFamily("sans");
    pGroup->setFontSize(20.0);
    pGroup->setVTextAnchor(Text::ANCHOR_BOTTOM);
    // add a rectangle to the group
    pRectangle=pGroup->createRectangle();
    pRectangle->setCoordinates(RelAbsVector(0.0,0.0),RelAbsVector(0.0,0.0),RelAbsVector(0.0,0.0));
    pRectangle->setSize(RelAbsVector(0.0,100.0),RelAbsVector(0.0,100.0));
    // add a text element to the group
    pText=pGroup->createText();
    pText->setText(text);
    y+=HEIGHT*1.2;
    // create a graphical object with the correct bounding box
    // centered lowercase letters
    os.str("");
    os << "graphical_object_" << object_index++;
    pObject=pLayout->createAdditionalGraphicalObject();
    pObject->setId(os.str());
    // we set the bound
    p=Point(x,y);
    bb.setPosition(&p);
    bb.setDimensions(&d);
    pObject->setBoundingBox(&bb);
    // we create a style for the object
    pStyle=pRenderInfo->createStyle("style_for_"+os.str());
    pStyle->addId(pObject->getId());
    pGroup=pStyle->getGroup();
    pGroup->setStroke("#000000");
    pGroup->setStrokeWidth(1.0);
    pGroup->setFontFamily("monospace");
    pGroup->setFontSize(20.0);
    pGroup->setVTextAnchor(Text::ANCHOR_TOP);
    // add a rectangle to the group
    pRectangle=pGroup->createRectangle();
    pRectangle->setCoordinates(RelAbsVector(0.0,0.0),RelAbsVector(0.0,0.0),RelAbsVector(0.0,0.0));
    pRectangle->setSize(RelAbsVector(0.0,100.0),RelAbsVector(0.0,100.0));
    // add a text element to the group
    pText=pGroup->createText();
    pText->setText(text);
    y+=HEIGHT*1.2;

    // create a graphical object with the correct bounding box
    // right alligned capital letters
    os.str("");
    os << "graphical_object_" << object_index++;
    pObject=pLayout->createAdditionalGraphicalObject();
    pObject->setId(os.str());
    // we set the bound
    p=Point(x,y);
    bb.setPosition(&p);
    bb.setDimensions(&d);
    pObject->setBoundingBox(&bb);
    // we create a style for the object
    pStyle=pRenderInfo->createStyle("style_for_"+os.str());
    pStyle->addId(pObject->getId());
    pGroup=pStyle->getGroup();
    pGroup->setStroke("#000000");
    pGroup->setStrokeWidth(1.0);
    pGroup->setFontFamily("monospace");
    pGroup->setFontSize(20.0);
    pGroup->setVTextAnchor(Text::ANCHOR_MIDDLE);
    // add a rectangle to the group
    pRectangle=pGroup->createRectangle();
    pRectangle->setCoordinates(RelAbsVector(0.0,0.0),RelAbsVector(0.0,0.0),RelAbsVector(0.0,0.0));
    pRectangle->setSize(RelAbsVector(0.0,100.0),RelAbsVector(0.0,100.0));
    // add a text element to the group
    pText=pGroup->createText();
    pText->setText(text);
    y+=HEIGHT*1.2;
    // create a graphical object with the correct bounding box
    // right alligned lowercase letters
    os.str("");
    os << "graphical_object_" << object_index++;
    pObject=pLayout->createAdditionalGraphicalObject();
    pObject->setId(os.str());
    // we set the bound
    p=Point(x,y);
    bb.setPosition(&p);
    bb.setDimensions(&d);
    pObject->setBoundingBox(&bb);
    // we create a style for the object
    pStyle=pRenderInfo->createStyle("style_for_"+os.str());
    pStyle->addId(pObject->getId());
    pGroup=pStyle->getGroup();
    pGroup->setStroke("#000000");
    pGroup->setStrokeWidth(1.0);
    pGroup->setFontFamily("monospace");
    pGroup->setFontSize(20.0);
    pGroup->setVTextAnchor(Text::ANCHOR_BOTTOM);
    // add a rectangle to the group
    pRectangle=pGroup->createRectangle();
    pRectangle->setCoordinates(RelAbsVector(0.0,0.0),RelAbsVector(0.0,0.0),RelAbsVector(0.0,0.0));
    pRectangle->setSize(RelAbsVector(0.0,100.0),RelAbsVector(0.0,100.0));
    // add a text element to the group
    pText=pGroup->createText();
    pText->setText(text);

    y+=HEIGHT*1.2;
    // create a graphical object with the correct bounding box
    // centered lowercase letters
    os.str("");
    os << "graphical_object_" << object_index++;
    pObject=pLayout->createAdditionalGraphicalObject();
    pObject->setId(os.str());
    // we set the bound
    p=Point(x,y);
    bb.setPosition(&p);
    bb.setDimensions(&d);
    pObject->setBoundingBox(&bb);
    // we create a style for the object
    pStyle=pRenderInfo->createStyle("style_for_"+os.str());
    pStyle->addId(pObject->getId());
    pGroup=pStyle->getGroup();
    pGroup->setStroke("#000000");
    pGroup->setStrokeWidth(1.0);
    pGroup->setFontFamily("serif");
    pGroup->setFontSize(20.0);
    pGroup->setVTextAnchor(Text::ANCHOR_TOP);
    // add a rectangle to the group
    pRectangle=pGroup->createRectangle();
    pRectangle->setCoordinates(RelAbsVector(0.0,0.0),RelAbsVector(0.0,0.0),RelAbsVector(0.0,0.0));
    pRectangle->setSize(RelAbsVector(0.0,100.0),RelAbsVector(0.0,100.0));
    // add a text element to the group
    pText=pGroup->createText();
    pText->setText(text);
    y+=HEIGHT*1.2;

    // create a graphical object with the correct bounding box
    // right alligned capital letters
    os.str("");
    os << "graphical_object_" << object_index++;
    pObject=pLayout->createAdditionalGraphicalObject();
    pObject->setId(os.str());
    // we set the bound
    p=Point(x,y);
    bb.setPosition(&p);
    bb.setDimensions(&d);
    pObject->setBoundingBox(&bb);
    // we create a style for the object
    pStyle=pRenderInfo->createStyle("style_for_"+os.str());
    pStyle->addId(pObject->getId());
    pGroup=pStyle->getGroup();
    pGroup->setStroke("#000000");
    pGroup->setStrokeWidth(1.0);
    pGroup->setFontFamily("serif");
    pGroup->setFontSize(20.0);
    pGroup->setVTextAnchor(Text::ANCHOR_MIDDLE);
    // add a rectangle to the group
    pRectangle=pGroup->createRectangle();
    pRectangle->setCoordinates(RelAbsVector(0.0,0.0),RelAbsVector(0.0,0.0),RelAbsVector(0.0,0.0));
    pRectangle->setSize(RelAbsVector(0.0,100.0),RelAbsVector(0.0,100.0));
    // add a text element to the group
    pText=pGroup->createText();
    pText->setText(text);
    y+=HEIGHT*1.2;
    // create a graphical object with the correct bounding box
    // right alligned lowercase letters
    os.str("");
    os << "graphical_object_" << object_index++;
    pObject=pLayout->createAdditionalGraphicalObject();
    pObject->setId(os.str());
    // we set the bound
    p=Point(x,y);
    bb.setPosition(&p);
    bb.setDimensions(&d);
    pObject->setBoundingBox(&bb);
    // we create a style for the object
    pStyle=pRenderInfo->createStyle("style_for_"+os.str());
    pStyle->addId(pObject->getId());
    pGroup=pStyle->getGroup();
    pGroup->setStroke("#000000");
    pGroup->setStrokeWidth(1.0);
    pGroup->setFontFamily("serif");
    pGroup->setFontSize(20.0);
    pGroup->setVTextAnchor(Text::ANCHOR_BOTTOM);
    // add a rectangle to the group
    pRectangle=pGroup->createRectangle();
    pRectangle->setCoordinates(RelAbsVector(0.0,0.0),RelAbsVector(0.0,0.0),RelAbsVector(0.0,0.0));
    pRectangle->setSize(RelAbsVector(0.0,100.0),RelAbsVector(0.0,100.0));
    // add a text element to the group
    pText=pGroup->createText();
    pText->setText(text);

    // write the document
    SBMLWriter writer = SBMLWriter();
    std::ostringstream fos;
    fos << "text_vertical_alignment_different_fonts_l" << level << "v" << version << ".xml";
    writer.writeSBML(pDocument,fos.str());
    delete pDocument;
}
