/* Copyright 2010 Ralph Gauges
 *
 * This library is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation.  A copy of the license agreement is
 * provided in the file named "LICENSE.txt" included with this software
 * distribution.  It is also available online at
 * http://www.gnu.org/licenses/lgpl.html
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with this library; if not, write to the Free Software Foundation,
 * Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA.
 *
 * The original code contained here was initially developed by:
 *
 *     Ralph Gauges
 *     Group for the modeling of biological processes 
 *     University of Heidelberg
 *     Im Neuenheimer Feld 267
 *     69120 Heidelberg
 *     Germany
 *
 *     mailto:ralph.gauges@bioquant.uni-heidelberg.de
 *
 * Contributor(s):
 *
 */
#include <sbml/SBMLDocument.h>
#include <sbml/SBMLWriter.h>
#include <sbml/Model.h>
#include <sbml/layout/Layout.h>
#include <sbml/layout/BoundingBox.h>
#include <sbml/layout/Point.h>
#include <sbml/layout/Dimensions.h>
#include <sbml/layout/GraphicalObject.h>
#include <sbml/layout/render/LocalRenderInformation.h>
#include <sbml/layout/render/GlobalRenderInformation.h>
#include <sbml/layout/render/LocalStyle.h>
#include <sbml/layout/render/GlobalStyle.h>
#include <sbml/layout/render/Rectangle.h>
#include <sbml/layout/render/Image.h>
#include <sbml/layout/render/Polygon.h>
#include <sbml/layout/render/RenderPoint.h>

#include <cmath>
#include <sstream>
#include <string>
#include <stdlib.h>
#include <iostream>

using namespace std;

/**
 * This example draws three rectangles.
 * the first rectangle contains a rotated filled rectangle,
 * the second contains a rotated image
 * and the third contains a rotated polygon
 */

int main(int argc,char** argv)
{
    // we define a transformation matrix for a 30 degree rotation
    double angle=10.0/180.0*M_PI;
    double transformation2D[6]={cos(angle),sin(angle),-sin(angle),cos(angle),0.0,0.0};
    unsigned int level=SBML_DEFAULT_LEVEL;
    unsigned int version=SBML_DEFAULT_VERSION;
    SBMLDocument* pDocument=NULL;
    if(argc != 1 && argc != 3)
    {
        std::cout << "usage: " << argv[0] << " [LEVEL VERSION]" << std::endl;
        std::cout << "e.g. " << argv[0] << " 2 3" << std::endl;
        std::cout << "or " << argv[0] << std::endl;
        return 0;
    }
    else
    {
       if(argc == 1)
       {
           pDocument=new SBMLDocument(level,version);
       }
       else
       {
         level=strtol(argv[1],NULL,10); 
         version=strtol(argv[2],NULL,10); 
         try{
           pDocument=new SBMLDocument(level,version);
           if(!pDocument->hasValidLevelVersionNamespaceCombination())
           {
             std::cerr << "Invalid Level and/or version. Aborting." << std::endl;
             return 1;
           }
         }
         catch(...)
         {
            std::cerr << "Invalid Level and/or version. Aborting." << std::endl;
            return 1;
         }
       }

    }
    assert(pDocument != NULL);
    Model* pModel=pDocument->createModel("Model_1");
    assert(pModel != NULL);
    if(level > 2)
    {
       UnitDefinition* pUDef=pModel->createUnitDefinition();
       pUDef->setId("squaremetre");
       pUDef->setName("squaremetre");
       Unit* pUnit=pUDef->createUnit();
       pUnit->setKind(UNIT_KIND_METRE);
       pUnit->setMultiplier(1.0);
       pUnit->setScale(0);
       pUnit->setExponent(2.0);
       pModel->setTimeUnits("second"); 
       pModel->setSubstanceUnits("mole"); 
       pModel->setVolumeUnits("litre"); 
       pModel->setAreaUnits("squaremetre"); 
       pModel->setLengthUnits("metre"); 
       pModel->setExtentUnits("mole"); 
    }

    // now we create a new layout
    Layout* pLayout=pModel->createLayout();
    pLayout->setId("TransformationTest");
    const double MARGIN_X=5.0;
    const double MARGIN_Y=5.0;
    const double V_SPACE=10.0;
    // draw each line in a 30x30 box
    const double SIDE_LARGE=800;
    const double SIDE_SMALL=400;
    ostringstream os;
    unsigned int object_index=1;
    // the dimensions of the whole layout will be
    Dimensions dim(2*MARGIN_X+SIDE_LARGE,2*MARGIN_Y+3*SIDE_SMALL+2*V_SPACE);
    pLayout->setDimensions(&dim);
    // create the render information
    LocalRenderInformation* pRenderInfo=pLayout->createLocalRenderInformation();
    pRenderInfo->setId("TransformationRenderInfo");
    
    // first rectangle
    // create a graphical object with the correct bounding box
    os.str("");
    os << "graphical_object_" << object_index++;
    GraphicalObject* pObject=pLayout->createAdditionalGraphicalObject();
    pObject->setId(os.str());
    // we set the bounding box
    BoundingBox bb;
    Dimensions d(SIDE_LARGE,SIDE_SMALL);
    Point p(MARGIN_X,MARGIN_Y);
    bb.setPosition(&p);
    bb.setDimensions(&d);
    pObject->setBoundingBox(&bb);
    // we create a style for the object
    LocalStyle* pStyle=pRenderInfo->createStyle("style_for_"+os.str());
    pStyle->addId(pObject->getId());
    Group* pGroup=pStyle->getGroup();
    pGroup->setStroke("#000000");
    pGroup->setStrokeWidth(2.0);
    // add an rectangle to the group
    Rectangle* pRectangle=pGroup->createRectangle();
    pRectangle->setCoordinates(RelAbsVector(0.0,0.0),RelAbsVector(0.0,0.0),RelAbsVector(0.0,0.0));
    pRectangle->setSize(RelAbsVector(0.0,100),RelAbsVector(0.0,100));
    // add another rectangle that does not fill the whole bounding box and
    // rotate it
    pRectangle=pGroup->createRectangle();
    pRectangle->setCoordinates(RelAbsVector(0.0,20.0),RelAbsVector(0.0,20.0),RelAbsVector(0.0,0.0));
    pRectangle->setSize(RelAbsVector(0.0,60),RelAbsVector(0.0,60));
    pRectangle->setFillColor("#FF0000");
    pRectangle->setMatrix2D(transformation2D);

    
    // second rectangle
    // create a graphical object with the correct bounding box
    os.str("");
    os << "graphical_object_" << object_index++;
    pObject=pLayout->createAdditionalGraphicalObject();
    pObject->setId(os.str());
    // we set the bounding box
    d=Dimensions(SIDE_LARGE,SIDE_SMALL);
    p=Point(MARGIN_X,MARGIN_Y+SIDE_SMALL+V_SPACE);
    bb.setPosition(&p);
    bb.setDimensions(&d);
    pObject->setBoundingBox(&bb);
    // we create a style for the object
    pStyle=pRenderInfo->createStyle("style_for_"+os.str());
    pStyle->addId(pObject->getId());
    pGroup=pStyle->getGroup();
    pGroup->setStroke("#000000");
    pGroup->setStrokeWidth(2.0);
    // add a rectangle to the group
    pRectangle=pGroup->createRectangle();
    pRectangle->setCoordinates(RelAbsVector(0.0,0.0),RelAbsVector(0.0,0.0),RelAbsVector(0.0,0.0));
    pRectangle->setSize(RelAbsVector(0.0,100),RelAbsVector(0.0,100));
    // add an image
    Image* pImage=pGroup->createImage();
    pImage->setCoordinates(RelAbsVector(0.0,20.0),RelAbsVector(0.0,20.0),RelAbsVector(0.0,0.0));
    pImage->setDimensions(RelAbsVector(0.0,60),RelAbsVector(0.0,60));
    pImage->setImageReference("bird_small.png");
    pImage->setMatrix2D(transformation2D);
     
    // third rectangle
    // create a graphical object with the correct bounding box
    os.str("");
    os << "graphical_object_" << object_index++;
    pObject=pLayout->createAdditionalGraphicalObject();
    pObject->setId(os.str());
    // we set the bounding box
    d=Dimensions(SIDE_LARGE,SIDE_SMALL);
    p=Point(MARGIN_X,MARGIN_Y+2*SIDE_SMALL+2*V_SPACE);
    bb.setPosition(&p);
    bb.setDimensions(&d);
    pObject->setBoundingBox(&bb);
    // we create a style for the object
    pStyle=pRenderInfo->createStyle("style_for_"+os.str());
    pStyle->addId(pObject->getId());
    pGroup=pStyle->getGroup();
    pGroup->setStroke("#000000");
    pGroup->setStrokeWidth(2.0);
    // add an rectangle to the group
    pRectangle=pGroup->createRectangle();
    pRectangle->setCoordinates(RelAbsVector(0.0,0.0),RelAbsVector(0.0,0.0),RelAbsVector(0.0,0.0));
    pRectangle->setSize(RelAbsVector(0.0,100),RelAbsVector(0.0,100));
    Polygon* pPolygon=pGroup->createPolygon();
    // create the segments for the polygon
    // first segment
    RenderPoint* pP=pPolygon->createPoint();
    RelAbsVector x(0.0,80.0);
    RelAbsVector y(0.0,40.0);
    pP->setCoordinates(x,y);
    x.setRelativeValue(32.0);
    y.setRelativeValue(80.0);
    pP=pPolygon->createPoint();
    pP->setCoordinates(x,y);

    // second segment
    x.setRelativeValue(50.0);
    y.setRelativeValue(20.0);
    pP=pPolygon->createPoint();
    pP->setCoordinates(x,y);

    // third segment
    x.setRelativeValue(68.0);
    y.setRelativeValue(80.0);
    pP=pPolygon->createPoint();
    pP->setCoordinates(x,y);

    // fourth segment
    x.setRelativeValue(20.0);
    y.setRelativeValue(40.0);
    pP=pPolygon->createPoint();
    pP->setCoordinates(x,y);
    
    pPolygon->setFillColor("#FF0000");
    pPolygon->setMatrix2D(transformation2D);

     // write the document
    SBMLWriter writer = SBMLWriter();
    std::ostringstream fos;
    fos << "rotate_l" << level << "v" << version << ".xml";
    writer.writeSBML(pDocument,fos.str());
    delete pDocument;
}
